
#include "Ball.h"

Ball::Ball(const orxCHAR* key){
	this->oBallObject = orxNULL;
	this->vIntersectionPoint = orxVECTOR_0;
	this->vPos = orxVECTOR_0;
	this->vPreviousPos = orxVECTOR_0;
	this->iBallDirection = -1;
	this->vVelocityConstant = orxVECTOR_0;
	this->fHorizontal = 0.f;
	this->fVertical = 0.f;
        this->iLastTouch = -1;
        this->bMegaStroke = false;

	Init(key);
}

Ball::~Ball(){
	if(this->oBallObject != orxNULL){
		orxObject_SetLifeTime(oBallObject, orxFLOAT_0);
		this->oBallObject = orxNULL;
	}
}

orxSTATUS Ball::Init(const orxCHAR* key){
	this->oBallObject = orxObject_CreateFromConfig(key);
	if(this->oBallObject == orxNULL){
		orxLOG("Init()-<%s> nenalezen!", key);
		return orxSTATUS_FAILURE;
	}

	this->sName = orxObject_GetName(this->oBallObject);

	orxConfig_PushSection("BallParticle");
		orxConfig_GetVector("VelocityConstant", &vVelocityConstant);
	orxConfig_PopSection();
        

	Disable();

	return orxSTATUS_SUCCESS;
}

void Ball::CalculateIntersection(){
	if(oBallObject != orxNULL){
		orxVECTOR step = orxVECTOR_0;
		orxVECTOR difference = orxVECTOR_0;

		orxObject_GetPosition(oBallObject, &vPos);

		difference.fX = vPos.fX - vPreviousPos.fX;
		difference.fY = vPos.fY - vPreviousPos.fY;

		if(difference.fY > 0 && difference.fX > 0){
			//down and right
			iBallDirection = 0;
		}else if(difference.fY < 0 && difference.fX < 0){
			//up and left
			iBallDirection = 1;
		}else if(difference.fY > 0 && difference.fX < 0){
			//down and left
			iBallDirection = 1;
		}else if(difference.fY < 0 && difference.fX > 0){
			//up and right
			iBallDirection = 0;
		}else if(difference.fX == 0 && difference.fY == 0){
			iBallDirection = -1;
		}

                //calculate intersection with edges of playground
		if(iBallDirection != -1){
			step.fY = orxMath_Abs(difference.fY/difference.fX);

			fHorizontal = difference.fX == 0.f ? 0.f : (difference.fX > 0.f ? 1.f : -1.f);
			fVertical = difference.fY == 0.f ? 0.f : (difference.fY > 0.f ? 1.f : -1.f);
	
			for(int k = 1; ;k++){
				if(DistFromOrigin(vPos.fX+k*fHorizontal, vPos.fY+k*step.fY*fVertical) > 320.f){
					vIntersectionPoint.fX = vPos.fX+k*fHorizontal; 
					vIntersectionPoint.fY = vPos.fY+k*step.fY*fVertical; 
					break;
				}
			}
		}
	

		vPreviousPos = vPos;
	}
}

void Ball::IncreaseSpeed(float power){
        orxVECTOR speed;
        orxObject_GetSpeed(oBallObject, &speed);
        
        if(bMegaStroke){
            power = MEGA_POWER;
            MegaStroke(false);
        }
        
        if(power == 0.f){
            speed.fX -= vVelocityConstant.fX*fHorizontal;
            speed.fY -= vVelocityConstant.fY*fVertical;
        }else{
            speed.fX -= power*fHorizontal;
            speed.fY -= power*fVertical;
        }
        orxObject_SetSpeed(oBallObject, &speed);

        orxObject_AddSound(oBallObject, "BallCollisionSound");
}

void Ball::DecreaseSpeed(){
        orxVECTOR speed;
        orxObject_GetSpeed(oBallObject, &speed);
        speed.fX += vVelocityConstant.fX*fHorizontal;
        speed.fY += vVelocityConstant.fY*fVertical;
        orxObject_SetSpeed(oBallObject, &speed);

        orxObject_AddSound(oBallObject, "BallCollisionSound");
}

void Ball::MegaStroke(bool enable){
    bMegaStroke = enable;
    orxVECTOR rgb;
    orxCOLOR color;
    if(enable){ 
        rgb.fR = 255.f;
        rgb.fG = 0.f;
        rgb.fB = 0.f;           
        orxColor_SetRGB(&color, &rgb);
        orxColor_SetAlpha(&color, 1.0f);
        orxObject_SetColor(oBallObject, &color);
    }else{
        rgb.fR = 51.f;
        rgb.fG = 0.f;
        rgb.fB = 255.f;       
        orxColor_SetRGB(&color, &rgb);
        orxColor_SetAlpha(&color, 1.0f);
        orxObject_SetColor(oBallObject, &color);
    }
}

int Ball::Update(){
	float lifetime = orxObject_GetLifeTime(oBallObject);
	if(lifetime < 1.0f && lifetime > 0.f){
		return 1;	//return OBJECT_STATUS::DEATH
	}

	return 2;	//return OBJECT_STATUS::NOTHING
}

void Ball::Enable(){
	orxObject_Enable(oBallObject, orxTRUE);
}

void Ball::Disable(){
	orxObject_Enable(oBallObject, orxFALSE);
}

void Ball::Reload(){
	orxObject_SetLifeTime(oBallObject, orxFLOAT_0);
	oBallObject = orxObject_CreateFromConfig("BallParticle");
}

void Ball::NormalMotion(){
	orxVECTOR ballSpeed;
	orxVECTOR motionSpeed;

	orxObject_GetSpeed(oBallObject, &ballSpeed);
	orxVector_Set(&motionSpeed, ballSpeed.fX/0.5f, ballSpeed.fY/0.5f, 0.0f);
	orxObject_SetSpeed(oBallObject, &motionSpeed);
}

void Ball::SlowMotion(){
	orxVECTOR ballSpeed;
	orxVECTOR motionSpeed;

	orxObject_GetSpeed(oBallObject, &ballSpeed);
	orxVector_Set(&motionSpeed, ballSpeed.fX*0.5f, ballSpeed.fY*0.5f, 0.0f);
	orxObject_SetSpeed(oBallObject, &motionSpeed);
}

float Ball::DistFromOrigin(float x, float y){
	return orxMath_Sqrt(x*x+y*y);
}

void Ball::SetSpeed(float sx, float sy){
	orxVECTOR temp = orxVECTOR_0;
	temp.fX = sx;
	temp.fY = sy;
	orxObject_SetSpeed(oBallObject, &temp);
}

void Ball::SetPosition(float px, float py){
	orxVECTOR temp = orxVECTOR_0;
	temp.fX = px;
	temp.fY = py;
	orxObject_SetPosition(oBallObject, &temp);
}

void Ball::SetPreviousPos(float px, float py){
	orxVECTOR temp = orxVECTOR_0;
	temp.fX = px;
	temp.fY = py;
	orxObject_SetSpeed(oBallObject, &temp);
}

void Ball::SetIntersection(float ix, float iy){
	orxVECTOR temp = orxVECTOR_0;
	temp.fX = ix;
	temp.fY = iy;

	vIntersectionPoint = temp;
}

const orxVECTOR Ball::GetSpeed(){
	orxVECTOR temp = orxVECTOR_0;
	orxObject_GetSpeed(oBallObject, &temp);
	return temp;
}

const orxVECTOR Ball::GetIntersection(){
		return vIntersectionPoint;
}
