#ifndef _CONTROL_H
#define _CONTROL_H

#include <iostream>
#include <map>

#include "orx.h"
#include "Player.h"
#include "AI.h"
#include "Ball.h"
#include "Border.h"
#include "TimeBar.h"

typedef enum{
	DEATH = 1,
	NOTHING,
        ACTIVATE_BONUS
}OBJECT_STATUS;

typedef enum{
        BALL_SLOW_MOTION = 0,
        CHANGE_SIDES,
        INVISIBILITY,
        HUGE_PADDLE,
        SMALL_PADDLE,
        MAGNETIC_PADDLE,        //it is not implemented
        MEGA_STROKE,
        RECOVER_BORDERS,
        PLAY_YOURSELF,
        BONUS_NONE
}BONUS;

typedef enum{
    PLAYER_LEFT,
    PLAYER_RIGHT
}BONUS_INITIATOR;

class Control{
private:      
        Paddle                          *pstPlayerLeft;
        Paddle                          *pstPlayerRight;
        
        Ball				*pstBall;
        Border				*pstBorderLeft;
        Border                          *pstBorderRight;
        
        TimeBar                         *pstBonusBar1;
        TimeBar                         *pstBonusBar2;
        TimeBar                         *pstPointBar1;
        TimeBar                         *pstPointBar2;
        
        orxOBJECT                       *pstCastLContainer;
        orxOBJECT                       *pstCastRContainer;

        static orxCLOCK                 *bonusClk;       

	OBJECT_STATUS			pstPlayerLeftStatus;
	OBJECT_STATUS			pstPlayerRightStatus;
	OBJECT_STATUS			pstBallStatus;
        
        static std::map<BONUS, orxOBJECT*> bonuses;
        bool                            bonusTimerActive;
        bool                            automaticBonusActivation;

	int				bGameWinner;	//1=vyhral hrac vpravo, -1=vyhral hrac vlevo, 0=nikdo zatim nevyhral

	static void	orxFASTCALL	StartCountDown(const orxCLOCK_INFO *_pstClockInfo, void *_pstContext);
        static void     orxFASTCALL     CreateBonus(const orxCLOCK_INFO *_pstClockInfo, void *_pstContext);
public:
	Control();
	~Control();

	orxSTATUS			CreateSingleGame();
	orxSTATUS			CreateMultiGame();
	orxSTATUS			CreateAIGame();
        void                            CreateUI();

	void				UpdateSingleInput();
	void				UpdateMultiInput();

	int				UpdateGame(const orxCLOCK_INFO *_pstClockInfo);

	void				DetectedCollisions(const std::string recipient, const std::string sender);
	void				UnDetectedCollisions(const std::string recipient, const std::string sender);

	void				EndCountDown(const orxEVENT *_pstEvent);
        
        void                            ActivateBonus(BONUS bonus, BONUS_INITIATOR ini);
        void                            DeactivateBonus(BONUS bonus, BONUS_INITIATOR ini);

	int				GetWinner(){ return bGameWinner;};
	Ball*				GetBall(){ return pstBall;};
};
#endif

