
#include "Enviroment.h"

//Static variables initialization
Enviroment* Enviroment::eInstance = NULL;
orxOBJECT* Enviroment::pstScene = orxNULL;
orxVIEWPORT* Enviroment::pstViewport = orxNULL;
orxS32 Enviroment::ss32ConfigID = 0;
APP_STATUS Enviroment::pstAppStatus = NONE;

Enviroment::Enviroment(){
	this->pstGameStatus = PAUSE;
	this->bMenuCreated = false;
	this->bMenuSingleSelected = false;
	this->bMenuMultiSelected = false;
	this->bMenuQuitSelected = false;
	this->fIntroDelay = 8.f;
        this->pstQuitApp = orxSTATUS_SUCCESS;
        this->pstGameControl = NULL;
        this->menuSelectSound = orxNULL;
        this->oMenuSingleObject = orxNULL;
        this->oMenuMultiObject = orxNULL;
        this->oMenuQuitObject = orxNULL;
}

Enviroment::~Enviroment(){   
}

Enviroment* Enviroment::GetInstance(){
	if(eInstance == NULL)
		eInstance = new Enviroment();
	
	return eInstance;
}

//Delete everything
void Enviroment::DeleteInstance(){
        DestroyWorld();
        
        if(pstViewport != orxNULL){
            orxViewport_Delete(pstViewport);
            pstViewport = orxNULL;
        }
        if(pstScene != orxNULL){
            orxObject_SetLifeTime(pstScene, orxFLOAT_0);
            pstScene = orxNULL;
        }
        if(menuSelectSound != orxNULL){
            orxSound_Delete(menuSelectSound);
            menuSelectSound = orxNULL;
        }
        if(oMenuSingleObject != orxNULL){
            orxObject_SetLifeTime(oMenuSingleObject, orxFLOAT_0);
            oMenuSingleObject = orxNULL;
        }
        if(oMenuMultiObject != orxNULL){
            orxObject_SetLifeTime(oMenuMultiObject, orxFLOAT_0);
            oMenuMultiObject = orxNULL;
        }
        if(oMenuQuitObject != orxNULL){
            orxObject_SetLifeTime(oMenuQuitObject, orxFLOAT_0);
            oMenuQuitObject = orxNULL;
        }

	if(eInstance != NULL){
		delete eInstance;
		eInstance = NULL;
	}
}

orxSTATUS Enviroment::LoadConfig(){
	orxSTATUS eResult = orxSTATUS_FAILURE;

	/* Deletes our scene */
	if(pstScene)
	{
	orxObject_Delete(pstScene);
	pstScene = orxNULL;
	}

	/* Deletes our viewport */
	if(pstViewport)
	{
	orxViewport_Delete(pstViewport);
	pstViewport = orxNULL;
	}

	/* Clears all config data */
	orxConfig_Clear();

	orxConfig_Load(orxConfig_GetMainFileName());
	orxConfig_SelectSection("Game");

	/* Is current ID valid? */
	if(ss32ConfigID < orxConfig_GetListCounter("ConfigList"))
	{
		const orxSTRING zConfigFile;

		/* Gets config file */
		zConfigFile = orxConfig_GetListString("ConfigList", ss32ConfigID);

		/* Can load it? */
		if((eResult = orxConfig_Load(zConfigFile)) != orxSTATUS_FAILURE)
		{
			/* Creates viewport */
			pstViewport = orxViewport_CreateFromConfig("Viewport");

			/* Creates our scene */
			pstScene = orxObject_CreateFromConfig("Scene");
		}
	}

	/* Done! */
	return eResult;
}

orxSTATUS Enviroment::PlayIntro(){
	orxSTATUS eResult = orxSTATUS_FAILURE;

	//set config id to intro
	this->ss32ConfigID = INI_INTRO;

	//set app status to intro
	this->pstAppStatus = INTRO;

	//load intro config
	eResult = LoadConfig();

	//load intro delay from config
	orxConfig_PushSection("Logo");
		this->fIntroDelay = orxConfig_GetFloat("IntroDelay");
	orxConfig_PopSection();
		
	//set up intro inputs
	eResult = orxInput_SelectSet("IntroInput");

	//done
	return eResult;
}

void Enviroment::EndIntro(const orxCLOCK_INFO *_pstClockInfo, void *_pstContext){
	pstAppStatus = MENU;
}

orxSTATUS Enviroment::StartGame(){
	orxSTATUS eResult = orxSTATUS_FAILURE;
        
        orxConfig_PushSection("Skip");
                bool skipIntro = orxConfig_GetBool("Intro");
        orxConfig_PopSection();
       
        //should be intro skipped?
        if(skipIntro == false){
            eResult = PlayIntro();

            //set up timer for ending intro
            orxClock_AddGlobalTimer(EndIntro, fIntroDelay, 1, orxNULL);
        }else{
            this->pstAppStatus = MENU;
            eResult = orxSTATUS_SUCCESS;
        }
        
        //create main clk(inputs update) and world clk(game update)
	clkMainClock = orxClock_FindFirst(orx2F(-1.0f), orxCLOCK_TYPE_CORE);
	clkWorldClock = orxClock_CreateFromConfig("WorldClock");
        
	//done
	return eResult;
}

orxSTATUS Enviroment::CreateMenu(){
	orxSTATUS eResult = orxSTATUS_FAILURE;

        orxConfig_PushSection("Skip");
                bool skipMenu = orxConfig_GetBool("Menu");
        orxConfig_PopSection();
        
        //should be menu skipped?
        if(skipMenu == false){
            this->ss32ConfigID = INI_MENU;

            eResult = LoadConfig();

            //set up menu inputs
            eResult = orxInput_SelectSet("MenuInput");
            
            //create menu
            this->oMenuSingleObject = orxObject_CreateFromConfig("MenuSingle");
            this->oMenuMultiObject = orxObject_CreateFromConfig("MenuMulti");
            this->oMenuQuitObject = orxObject_CreateFromConfig("MenuQuit");
            
            orxObject_Enable(oMenuSingleObject, orxTRUE);
            orxObject_Enable(orxObject_GetOwnedChild(oMenuSingleObject), orxTRUE);
            orxObject_Enable(oMenuMultiObject, orxFALSE);
            orxObject_Enable(orxObject_GetOwnedChild(oMenuMultiObject), orxFALSE);
            orxObject_Enable(oMenuQuitObject, orxFALSE);
            orxObject_Enable(orxObject_GetOwnedChild(oMenuQuitObject), orxFALSE);
            
            this->pstAppStatus = MENU;           
            this->menuSelectSound = orxSound_CreateFromConfig("MenuSelectSound");
            this->pstMenuStatus = SINGLE_PLAYER;
            this->bMenuCreated = true;
        }else{
            //when menu is skipped then the single mode will be initiated
            eResult = orxSTATUS_SUCCESS;
            this->CreateWorld(SINGLE);
        }

        //done
	return eResult;
}

orxSTATUS Enviroment::CreateWorld(GAME_TYPE type){
	orxSTATUS eResult = orxSTATUS_FAILURE;
        
	this->ss32ConfigID = INI_GAME;

	//game type
	this->pstGameType = type;

	//main class for control the game 
	this->pstGameControl = new Control();

	eResult = LoadConfig();

	this->pstAppStatus = GAME;

	switch(type){
	//single player game mode
	case SINGLE:{
		//create objects for single player game
		eResult = pstGameControl->CreateSingleGame();
		break;
		}
	//multi player game mode
	case MULTI:{
		//create objects for multi player game
		eResult = pstGameControl->CreateMultiGame();
		break;
		}

	//ai vs ai (it is not work properly)
	case ONLY_AI:{
		//create objects for ai vs ai game
		eResult = pstGameControl->CreateAIGame();
		break;
		}
        //bad game type
	default:
		eResult = orxSTATUS_FAILURE;
		break;
	};

        //show the countdown
	orxObject_CreateFromConfig("BlackLayer");
	orxOBJECT* countdown = orxObject_CreateFromConfig("CountDown");
	orxObject_SetCurrentAnim(countdown, "CountDownAnim");

        if(menuSelectSound != orxNULL){
            orxSound_Delete(menuSelectSound);
            menuSelectSound = orxNULL;
        }
        
        //done
	return eResult;
}

//delete game control
void Enviroment::DestroyWorld(){
    if(pstGameControl != NULL){
	delete pstGameControl;
	pstGameControl = NULL;
    }
}

void Enviroment::UpdateGlobalInput(){
        //it depends on the state of application
	switch(pstAppStatus){
	case GAME:{
		//if game isnt pause
		if(pstGameStatus == PLAYING){
			//it depends on type of the game
			switch(pstGameType){
			//Single
			case SINGLE:{
				pstGameControl->UpdateSingleInput();
				break;
				}
			//Multi
			case MULTI:{
				pstGameControl->UpdateMultiInput();
				break;
				}
			default:
				break;
			};
		}
		break;
		}
	//For skip the intro
	case INTRO:{
		if(orxInput_IsActive("Skip") && orxInput_HasNewStatus("Skip")){
			pstAppStatus = MENU;
			orxClock_RemoveGlobalTimer(EndIntro, fIntroDelay, orxNULL);
		}
		break;
		}	
	case MENU:{
		if(orxInput_IsActive("MenuUp") && orxInput_HasNewStatus("MenuUp")){
                        orxSound_Play(menuSelectSound);
                        //for cycling in menu
			pstMenuStatus = ((pstMenuStatus-1) == SINGLE_PLAYER) ? SINGLE_PLAYER : (((pstMenuStatus-1) < SINGLE_PLAYER) ? QUIT : MULTI_PLAYER); 
		}else if(orxInput_IsActive("MenuDown") && orxInput_HasNewStatus("MenuDown")){
                        orxSound_Play(menuSelectSound);
			pstMenuStatus = ((pstMenuStatus+1) == QUIT) ? QUIT : (((pstMenuStatus+1) > QUIT) ? SINGLE_PLAYER : MULTI_PLAYER); 
		}else if(orxInput_IsActive("MenuEnter") && orxInput_HasNewStatus("MenuEnter")){
			switch(pstMenuStatus){
			case SINGLE_PLAYER:
				CreateWorld(SINGLE);
				break;
			case MULTI_PLAYER:
                                CreateWorld(MULTI);
				break;
			case QUIT:
                                DestroyWorld();
                                pstAppStatus = QUIT_APP;
                                break;
			};
                        
                        //delete menu
                        if(oMenuSingleObject != orxNULL){
                            orxObject_SetLifeTime(oMenuSingleObject, orxFLOAT_0);
                            oMenuSingleObject = orxNULL;
                        }
                        if(oMenuMultiObject != orxNULL){
                            orxObject_SetLifeTime(oMenuMultiObject, orxFLOAT_0);
                            oMenuMultiObject = orxNULL;
                        }
                        if(oMenuQuitObject != orxNULL){
                            orxObject_SetLifeTime(oMenuQuitObject, orxFLOAT_0);
                            oMenuQuitObject = orxNULL;
                        }
		}else if(orxInput_IsActive("MenuAIStart") && orxInput_HasNewStatus("MenuAIStart")){
                    CreateWorld(ONLY_AI);       
                }
		break;
		}

	default:
		break;
	};
}

void Enviroment::UpdateMenu(){
	if(pstMenuStatus == SINGLE_PLAYER && bMenuSingleSelected == false){	
		bMenuSingleSelected = true;
		bMenuMultiSelected = false;
		bMenuQuitSelected = false;
                
                orxObject_Enable(oMenuSingleObject, orxTRUE);
                orxObject_Enable(orxObject_GetOwnedChild(oMenuSingleObject), orxTRUE);
                orxObject_Enable(oMenuMultiObject, orxFALSE);
                orxObject_Enable(orxObject_GetOwnedChild(oMenuMultiObject), orxFALSE);
                orxObject_Enable(oMenuQuitObject, orxFALSE);
                orxObject_Enable(orxObject_GetOwnedChild(oMenuQuitObject), orxFALSE);
	}else if(pstMenuStatus == MULTI_PLAYER && bMenuMultiSelected == false){	
		bMenuSingleSelected = false;
		bMenuMultiSelected = true;
		bMenuQuitSelected = false;
                
                orxObject_Enable(oMenuSingleObject, orxFALSE);
                orxObject_Enable(orxObject_GetOwnedChild(oMenuSingleObject), orxFALSE);
                orxObject_Enable(oMenuMultiObject, orxTRUE);
                orxObject_Enable(orxObject_GetOwnedChild(oMenuMultiObject), orxTRUE);
                orxObject_Enable(oMenuQuitObject, orxFALSE);
                orxObject_Enable(orxObject_GetOwnedChild(oMenuQuitObject), orxFALSE);
	}else if(pstMenuStatus == QUIT && bMenuQuitSelected == false){	
		bMenuSingleSelected = false;
		bMenuMultiSelected = false;
		bMenuQuitSelected = true;
                
                orxObject_Enable(oMenuSingleObject, orxFALSE);
                orxObject_Enable(orxObject_GetOwnedChild(oMenuSingleObject), orxFALSE);
                orxObject_Enable(oMenuMultiObject, orxFALSE);
                orxObject_Enable(orxObject_GetOwnedChild(oMenuMultiObject), orxFALSE);
                orxObject_Enable(oMenuQuitObject, orxTRUE);
                orxObject_Enable(orxObject_GetOwnedChild(oMenuQuitObject), orxTRUE);
	}
}

void Enviroment::UpdateAll(const orxCLOCK_INFO *_pstClockInfo){
	//it depends on the state of application
	switch(pstAppStatus){
	case MENU:
		UpdateMenu();
		break;
	case GAME:{
		if(pstGameStatus == PLAYING){
                    pstGameStatus = (GAME_STATUS)pstGameControl->UpdateGame(_pstClockInfo);
		}else if(pstGameStatus == END){
			switch(pstGameControl->GetWinner()){
			//who is the winner
			case 1:
				std::cout << "Player 1 won" << std::endl;
				break;
			case -1:
				std::cout << "Player 2 won" << std::endl;
				break;
			//nobody won
			case 0:
				std::cout << "Game was quit" << std::endl;
				break;

			default:
				break;
			}
                        DestroyWorld();
                        pstGameStatus = PAUSE;
                        CreateMenu();
		}

		break;
		}
            case QUIT_APP:
                pstQuitApp = orxSTATUS_FAILURE;
                break;

	default:
		break;
	};
}

void Enviroment::DetectedCollisions(const std::string recipient, const std::string sender){
	pstGameControl->DetectedCollisions(recipient, sender);
}

void Enviroment::UnDetectedCollisions(const std::string recipient, const std::string sender){
	pstGameControl->UnDetectedCollisions(recipient, sender);
}

