#ifndef _ENVIROMENT_H
#define _ENVIROMENT_H

#include "orx.h"
#include <string>
#include <iostream> // temporary

#include "Control.h"
#include "Tools.h"

//constants for ini loading
#define INI_INTRO 0
#define INI_MENU 1
#define INI_GAME 2

/*STATES*/
//Menu
enum MENU_STATUS{
	SINGLE_PLAYER = 1,
	MULTI_PLAYER,
	QUIT
};

//Type of game
enum GAME_TYPE{
	SINGLE = 1,
	ONLY_AI,        //doesnt work properly
	MULTI
};

//State of game
enum GAME_STATUS{
	PLAYING = 1,
	PAUSE,         
	END
};

//Application state
enum APP_STATUS{
	INTRO = 1,
	MENU,
	GAME,
	NONE,
        QUIT_APP
};

//Singleton class
class Enviroment{
private:
	/*Class object*/
	static Enviroment		*eInstance;

	/*Variables for change the scene*/
	static orxS32			ss32ConfigID;	//id ini file
	static orxVIEWPORT		*pstViewport;	//app window
	static orxOBJECT		*pstScene;	//objects in the scene
        
        orxSTATUS                       pstQuitApp;

	/*Class controlling the whole game*/
	Control				*pstGameControl;

	/*States*/
	static APP_STATUS		pstAppStatus;	//in which state the application is (menu, intro, game ...)
	GAME_TYPE			pstGameType;	//what kind of game the player choose (single, multi)
	GAME_STATUS			pstGameStatus;	//in which state the game is (playing, pause, end)
	MENU_STATUS			pstMenuStatus;	//what is selected (single, multi, end)

	/*Menu variables*/
	bool				bMenuCreated;		
	bool				bMenuSingleSelected;	
	bool				bMenuMultiSelected;		
	bool				bMenuQuitSelected;		
	float				fIntroDelay;		//How long the intro will be (its possible change the variable in ini file)
        orxSOUND*                       menuSelectSound;
        orxOBJECT*                      oMenuSingleObject;
        orxOBJECT*                      oMenuMultiObject;
        orxOBJECT*                      oMenuQuitObject;

	/*Clocks*/
	orxCLOCK*			clkMainClock;	//clk for input update
	orxCLOCK*			clkWorldClock;	//clk for game update 
        
	/*Private functions*/
        Enviroment();
	~Enviroment();
        
	orxSTATUS                       PlayIntro();
	static void orxFASTCALL         EndIntro(const orxCLOCK_INFO *_pstClockInfo, void *_pstContext);
	static orxINLINE orxSTATUS      LoadConfig();


	//static void orxFASTCALL         UpdateBorders(const orxCLOCK_INFO *_pstClockInfo, void *_pstContext);
public: 
	static Enviroment*  GetInstance();
	void			DeleteInstance();

	orxSTATUS		StartGame();
	orxSTATUS		CreateMenu();
	orxSTATUS		CreateWorld(GAME_TYPE type);      
        orxSTATUS               CheckStatus(){ return pstQuitApp;};
	void			DestroyWorld();

	void			UpdateGlobalInput();
	void			UpdateMenu();
	void			UpdateAll(const orxCLOCK_INFO *_pstClockInfo);

	void			DetectedCollisions(const std::string recipient, const std::string sender);
	void			UnDetectedCollisions(const std::string recipient, const std::string sender);

	void			SetGameStatus(GAME_STATUS status){ pstGameStatus = status;};
	
	orxCLOCK*		GetMainClock(){ return clkMainClock;};
	orxCLOCK*		GetWorldClock(){ return clkWorldClock;};
	const APP_STATUS	GetAppStatus(){ return pstAppStatus;};
	const GAME_STATUS	GetGameStatus(){ return pstGameStatus;};
	const bool		IsMenuCreated(){ return bMenuCreated;};
	Control*		GetControl(){ return pstGameControl;};
};

#endif