
#include "Paddle.h"
#include "Tools.h"

Paddle::Paddle(){
	this->fSpeed = 0.f;
	this->fMaxSpeed = 0.f;
	this->fAngle = 0.f;
        this->iSide = 0;
	this->iRadius = 320;
	this->iDirection = 0;
	this->iPoints = 0;
	this->oPaddleObject = orxNULL;
        this->oBonusObject = orxNULL;
        this->oIconBonusObject = orxNULL;
        this->bBonusTimerActivated = false;
        this->bNegativeBonus = false;
	this->vPaddlePos.fX = 0.0;
	this->vPaddlePos.fY = 0.0;
	this->vPaddlePos.fZ = 0.0;
        this->vBonusPos.fX = 0.f;
	this->vBonusPos.fY = 0.f;
	this->vBonusPos.fZ = 0.1;
        this->iBonus = 9;       //BONUS_NONE
        this->iBonusActivated = 9;      //BONUS_NONE
        this->sBonusName = "";
        this->bHuman = false;
        this->sMovement = orxSound_CreateFromConfig("PaddleMovementSound");
}

Paddle::~Paddle(){
	if(oPaddleObject != orxNULL){
		orxObject_SetLifeTime(oPaddleObject, orxFLOAT_0);
		oPaddleObject = orxNULL;
	}
        
        if(oBonusObject != orxNULL){
		orxObject_SetLifeTime(oBonusObject, orxFLOAT_0);
		oBonusObject = orxNULL;
	}
        if(oIconBonusObject != orxNULL){
		orxObject_SetLifeTime(oIconBonusObject, orxFLOAT_0);
		oIconBonusObject = orxNULL;
	}
        if(sMovement != orxNULL){
            orxSound_Delete(sMovement);
            sMovement = orxNULL;
        }
}

int Paddle::PlusPoint(){
	iPoints += 1;

	return iPoints;
}

void Paddle::Reload(){
        orxSound_Pause(sMovement);
	orxObject_SetLifeTime(oPaddleObject, orxFLOAT_0);
	oPaddleObject = orxObject_CreateFromConfig(sName);
	fAngle = 0.f;
	iDirection = 0;
        iBonusActivated = 9;
}

void Paddle::Enable(){
	orxObject_Enable(oPaddleObject, orxTRUE);
}

void Paddle::Disable(){
	orxObject_Enable(oPaddleObject, orxFALSE);
}

void Paddle::DisableSpawner(){
	orxSpawner_Enable(orxOBJECT_GET_STRUCTURE(oPaddleObject, SPAWNER), orxFALSE);
}

void Paddle::Movement(const orxCLOCK_INFO *_pstClockInfo){
        //speed increase
        if(iDirection != 0){
                fSpeed += 0.25f;

                if(fSpeed > fMaxSpeed){
                        fSpeed = fMaxSpeed;
                }
                
                orxSound_Play(sMovement);
        }else{
            orxSound_Pause(sMovement);
        }

        fAngle += fSpeed*iDirection*orxMATH_KF_DEG_TO_RAD; 
        vPaddlePos.fX = orxMath_Cos(fAngle)*iRadius;
        vPaddlePos.fY = orxMath_Sin(fAngle)*iRadius;

	if(oPaddleObject != orxNULL){
		orxObject_SetPosition(oPaddleObject, &vPaddlePos);
		orxObject_SetRotation(oPaddleObject, this->fAngle);
	}
}

void Paddle::SetDirection(int value){
	if(value == 0){
		fSpeed = 0.f;
	}

	iDirection = value;
}

void Paddle::SetColor(orxCOLOR* color){
    orxColor_SetAlpha(color, 1.0f);
    orxObject_SetColor(oPaddleObject, color);
}

orxCOLOR* Paddle::GetColor(){
    orxCOLOR color;  
    return orxObject_GetColor(oPaddleObject, &color);
}

void Paddle::SetBonus(int bonus, orxCHAR* name, bool show){
    if(iBonus != 9 && oBonusObject != orxNULL){
        orxObject_SetLifeTime(oBonusObject, orxFLOAT_0);
        oBonusObject = orxNULL;
    }
    
    iBonus = bonus;
      
    orxCHAR temp[25] = "";
    orxString_Print(temp, "Icon%s", name);

    sBonusName = temp;
    oBonusObject = orxObject_CreateFromConfig(sBonusName);
       
    if(show){
        orxObject_Enable(oBonusObject, orxTRUE);
        orxObject_AddSound(oPaddleObject, "BonusGainSound");
        orxObject_SetPosition(oBonusObject, &vBonusPos);
    }else{
        orxObject_Enable(oBonusObject, orxFALSE);
    }   
}

int Paddle::ActivateBonus(){       
    if(iBonus != 9){
        oIconBonusObject = orxObject_CreateFromConfig(orxObject_GetName(oBonusObject));
        if(oIconBonusObject != orxNULL){
            orxObject_Enable(oIconBonusObject, orxTRUE);

            orxVECTOR scale, new_pos;
            orxVector_Set(&scale, 2.68f, 1.419f, 0.f);

            if(iSide == -1) //left
                    orxVector_Set(&new_pos, -505.f, -290.f, 0.2f);
            else if(iSide == 1)     //right
                    orxVector_Set(&new_pos, 307.f, -290.f, 0.2f);

            orxObject_SetScale(oIconBonusObject, &scale);
            orxObject_SetPosition(oIconBonusObject, &new_pos);
        }
    }
    
    if(oBonusObject != orxNULL){
        orxObject_SetLifeTime(oBonusObject, orxFLOAT_0);
        oBonusObject = orxNULL;
    }
    
   
    bBonusTimerActivated = true;
    iBonusActivated = iBonus;
    iBonus = 9; //BONUS_NONE
    sBonusName = orxNULL;
    orxObject_AddSound(oPaddleObject, "BonusUseSound");
    return iBonusActivated;
}

void Paddle::NormalMotion(){
    fMaxSpeed = fMaxSpeed/0.5f;
}

void Paddle::SlowMotion(){
    fMaxSpeed = fMaxSpeed*0.5f;
}

void Paddle::Invisible(bool enable){
    orxCOLOR color;
    orxObject_GetColor(oPaddleObject, &color);
    
    if(enable)
        orxColor_SetAlpha(&color, 0.f);
    else
        orxColor_SetAlpha(&color, 1.f);
    
    orxObject_SetColor(oPaddleObject, &color);
}

void Paddle::Size(float size){
    orxVECTOR scale;
    orxVector_Set(&scale, 1.0f, size, 0.f);
    orxObject_SetScale(oPaddleObject, &scale);
}

void Paddle::BonusNone(){ 
    iBonusActivated = 9; 
    bBonusTimerActivated = false;
    orxObject_AddSound(oPaddleObject, "BonusLostSound");
    
    if(oIconBonusObject != orxNULL){
        orxObject_SetLifeTime(oIconBonusObject, orxFLOAT_0);
        oIconBonusObject = orxNULL;
    }
}




