#include "TimeBar.h"

TimeBar::TimeBar(orxVECTOR bg_position, orxVECTOR line_position, orxCOLOR color, orxOBJECT* bg, float scalex, float scaley, float seconds, bool charge){
    this->_timeLine = orxObject_CreateFromConfig("Bar");
    this->_background = bg;   
    this->_time = 0.02f;        //dont change it
    this->_width = scalex;
    this->_height = scaley;
    this->_delay = 0.f;
    this->_charge = charge;
    
    if(seconds < 0)
        seconds *= -1;
    
    this->_discharge = ((_width*18.f)/280.f)/seconds;    //18.f == 1s, if (discharge>18.f) => (time<1s), if(discharge<18.f) => (time>1s)
    
    orxObject_SetColor(_timeLine, &color);
    orxObject_SetPosition(_background, &bg_position); 
    orxObject_SetPosition(_timeLine, &line_position);
    
    orxVECTOR scale;
    orxVector_Set(&scale, _width, _height, 0.f);
    
    if(!charge){
        this->_currentTime = _width;
        this->_discharge *= -1;
        orxVector_Set(&scale, _width, _height, 0.f);        
    }else{
        this->_currentTime = 0.1f;
        this->_width = 0.1f;
        orxVector_Set(&scale, _width, _height, 0.f);
    }
    
    this->_width = scalex;
    orxObject_SetScale(_timeLine, &scale);    
}

TimeBar::TimeBar(orxVECTOR bg_position, orxVECTOR line_position, orxCOLOR color, orxOBJECT* bg, float scalex, float scaley, int parts, bool charge){
    this->_timeLine = orxObject_CreateFromConfig("Bar");
    this->_background = bg;   
    this->_time = 0.02f;        //dont change it
    this->_width = scalex;
    this->_height = scaley;
    this->_delay = 0.f;
    this->_charge = charge;
    
    this->_discharge = float(_width/parts);
    
    orxObject_SetColor(_timeLine, &color);
    orxObject_SetPosition(_background, &bg_position); 
    orxObject_SetPosition(_timeLine, &line_position);
    
    orxVECTOR scale;
    orxVector_Set(&scale, _width, _height, 0.f);
    
    if(!charge){
        this->_currentTime = _width;
        this->_discharge *= -1;
        orxVector_Set(&scale, _width, _height, 0.f);        
    }else{
        this->_currentTime = 0.1f;
        this->_width = 0.1f;
        orxVector_Set(&scale, _width, _height, 0.f);
    }
    
    this->_width = scalex;
    orxObject_SetScale(_timeLine, &scale);    
}

TimeBar::TimeBar(const TimeBar& orig){
}

TimeBar::~TimeBar() {
    if(_timeLine != orxNULL){
        orxObject_SetLifeTime(_timeLine, orxFLOAT_0);
        _timeLine = orxNULL;
    }
    if(_background != orxNULL){
       orxObject_SetLifeTime(_background, orxFLOAT_0);
       _background = orxNULL;
    }
}

bool TimeBar::Add(){  
   orxVECTOR scale; 
   _currentTime += _discharge;
   if(_currentTime > 0 && _currentTime < _width){      
       orxVector_Set(&scale, _currentTime, _height, 0.f);
       orxObject_SetScale(_timeLine, &scale);
       return false;
   }else{
        return true;
    }
}

bool TimeBar::Update(const orxCLOCK_INFO *_pstClockInfo){
    orxVECTOR scale;
    _delay += _pstClockInfo->fTickSize;
    
    if(_delay > _time){
        _delay = 0.f;
        _currentTime += _discharge; 
        if(_currentTime >= 0 && _currentTime <= _width){                 
            orxVector_Set(&scale, _currentTime, _height, 0.f);
            orxObject_SetScale(_timeLine, &scale);
            return false;
        }else{
            if(_currentTime < 0){
                _currentTime = 0.f;
            }else{
                _currentTime = _width;
            }
            return true;
        } 
    }else{
        return false;
    }
}

void TimeBar::SetFill(float percentage){
    if(percentage >= 0 && percentage <= 100){
        _currentTime = (percentage==0) ? 0.1f : (percentage/100)*_width;
        
        orxVECTOR scale;
        orxVector_Set(&scale, _currentTime, _height, 0.f);
        orxObject_SetScale(_timeLine, &scale);
    }
}

void TimeBar::SetSeconds(float seconds){
    this->_discharge = 18.f / seconds;
    if(!_charge)
        _discharge *= -1;
}

