#include "orx.h"
#include <string>
#include <iostream>

#include "Enviroment.h"

Enviroment *gameInstance;

orxSTATUS orxFASTCALL EventHandler(const orxEVENT *_pstEvent)
{
	switch(_pstEvent->eType){
	case orxEVENT_TYPE_PHYSICS:
		if(_pstEvent->eID == orxPHYSICS_EVENT_CONTACT_ADD){
			orxOBJECT* object_recipient = orxOBJECT(_pstEvent->hRecipient);
			orxOBJECT* object_sender = orxOBJECT(_pstEvent->hSender);

			std::string name_recipient(orxObject_GetName(object_recipient));
			std::string name_sender(orxObject_GetName(object_sender));
                        
                        if(name_sender.compare("BallParticle") == 0){
                            std::string temp = name_sender;
                            name_sender = name_recipient;
                            name_recipient = temp;
                        }
                        
			//Collision detection
			gameInstance->DetectedCollisions(name_recipient, name_sender);

		}else if(_pstEvent->eID == orxPHYSICS_EVENT_CONTACT_REMOVE){
			orxOBJECT* object_recipient = orxOBJECT(_pstEvent->hRecipient);
			orxOBJECT* object_sender = orxOBJECT(_pstEvent->hSender);

			std::string name_recipient(orxObject_GetName(object_recipient));
			std::string name_sender(orxObject_GetName(object_sender));

                        if(name_sender.compare("BallParticle") == 0){
                            std::string temp = name_sender;
                            name_sender = name_recipient;
                            name_recipient = temp;
                        }
                        
			//Objects dont collide anymore
			gameInstance->UnDetectedCollisions(name_recipient, name_sender);
		}
		break;

	case orxEVENT_TYPE_ANIM:
                //3-2-1 animation handling
		orxANIM_EVENT_PAYLOAD* EventPayload = (orxANIM_EVENT_PAYLOAD*)_pstEvent->pstPayload;
		if(EventPayload->zCustomEventName != orxNULL){
                        //if animation is back at one then start the game and stop the animation
			if(orxString_Compare(EventPayload->zCustomEventName, "one")==0){
				gameInstance->SetGameStatus(PLAYING);
				gameInstance->GetControl()->EndCountDown(_pstEvent);
			}
                }
                break;
	};

	return orxSTATUS_SUCCESS;
}

void orxFASTCALL Update(const orxCLOCK_INFO *_pstClockInfo, void *_pstContext){
	//if intro then do nothing
	if(gameInstance->GetAppStatus() == INTRO){
		return;
	//if menu and menu isnt created then create menu
	}else if(gameInstance->GetAppStatus() == MENU && gameInstance->IsMenuCreated() == false)
		gameInstance->CreateMenu();
	
	//Update all objects in game or menu
	gameInstance->UpdateAll(_pstClockInfo);
}

void orxFASTCALL InputUpdate(const orxCLOCK_INFO *_pstClockInfo, void *_pstContext){
	//Update inputs
	gameInstance->UpdateGlobalInput();
}

orxSTATUS orxFASTCALL Run(){
	return gameInstance->CheckStatus();
}

void orxFASTCALL Exit(){
    //delete everything
    if(gameInstance != NULL){
        gameInstance->DeleteInstance();
        gameInstance = NULL;
    }
}

orxSTATUS orxFASTCALL Init(){
	orxSTATUS eResult = orxSTATUS_FAILURE;
	
	//game instance
	gameInstance = Enviroment::GetInstance();

	//start game
	eResult = gameInstance->StartGame();

	//Clock creation
	orxCLOCK* mainClock = gameInstance->GetMainClock();
	orxCLOCK* worldClock = gameInstance->GetWorldClock();

	orxEvent_AddHandler(orxEVENT_TYPE_PHYSICS, EventHandler);
	orxEvent_AddHandler(orxEVENT_TYPE_ANIM, EventHandler);

	//Update inputs
	orxClock_Register(mainClock, InputUpdate, orxNULL, orxMODULE_ID_MAIN, orxCLOCK_PRIORITY_NORMAL);
	//Game update
	orxClock_Register(worldClock, Update, orxNULL, orxMODULE_ID_MAIN, orxCLOCK_PRIORITY_NORMAL);

	return eResult;
}

int main(int argc, char **argv){
	orx_Execute(argc, argv, Init, Run, Exit);
	return EXIT_SUCCESS;
}

#ifdef __orxWINDOWS__

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
  /* Executes orx */
  orx_WinExecute(Init, Run, Exit);

  /* Done! */
  return EXIT_SUCCESS;
}

#endif // __orxWINDOWS__
